/*
 * Copyright (c) 1980 Regents of the University of California.
 * All rights reserved.  The Berkeley software License Agreement
 * specifies the terms and conditions for redistribution.
 *
 * This module is believed to contain source code proprietary to AT&T.
 * Use and redistribution is subject to the Berkeley Software License
 * Agreement and your Software Agreement with AT&T (Western Electric).
 *
 *	@(#)config.h	1.3 (gritter) 12/18/00
 */

/*
 * Configurable settings for the ex editor.
 */

/*
 * Maximum screen size in visual mode.
 *
 * Because the routine "alloca" is not portable, TUBESIZE
 * bytes are allocated on the stack each time you go into visual
 * and then never freed by the system.  Thus if you have no terminals
 * which are larger than 24 * 80 you may well want to make TUBESIZE
 * smaller.  TUBECOLS should stay at 160 since this defines the maximum
 * length of opening on hardcopies and allows two lines of open on
 * terminals like adm3's (glass tty's) where it switches to pseudo
 * hardcopy mode when a line gets longer than 80 characters.
 */
#ifndef VMUNIX
#define TUBELINES       70      /* Number of screen lines for visual */
#define TUBECOLS        160     /* Number of screen columns for visual */
#define TUBESIZE        6000    /* Maximum screen size for visual */
#else	/* VMUNIX */
#define TUBELINES       70
#define TUBECOLS        160
#define TUBESIZE        7000    /* 70 * 100 */
#endif	/* VMUNIX */

/*
 * Intptr_t was introduced by SUSv2, and it is highly necessary to achieve
 * portability between various processors. It is a signed integer type
 * capable of holding pointers: sizeof(intptr_t) == sizeof(char *).
 *
 * Type  Environment Typical systems
 * int   IP16        PDP11, 80286
 * int   ILP32       Most VAX, M68k, IA32, SPARC
 * long  LP32        Some IA32 and M68k
 * long  LP64        64 bit mode of IA64, SPARC v9, and Alpha
 *
 * The argument to the sbrk() system call has this type.
 */
#ifdef	notdef
typedef int	intptr_t;
#endif

/*
 * The following types are specified by POSIX.1.
 */

/*
 * The sizeof operator evaluates to this type.
 */
#ifdef	notdef
typedef	int	size_t;
#endif

/*
 * Process identification numbers as returned by getpid(2).
 */
#ifdef	notdef
typedef int     pid_t;
#endif

/*
 * User identification numbers as returned by getuid(2).
 */
#ifdef	notdef
typedef int     uid_t;
#endif

/*
 * Time values as returned by time(2).
 */
#ifdef	notdef
typedef long    time_t;
#endif

/*
 * File descriptor offsets as returned by lseek(2).
 */
#ifdef	notdef
typedef long    off_t;
#endif

/*
 * Signed size type as returned by read(2) and write(2).
 */
#ifdef	notdef
typedef int     ssize_t;
#endif

/*
 * The __STDC__ macro is used to test for compliance with ANSI C. Some
 * compilers comply to this, but do not define it; the value is never
 * used within ex.
 */
#ifdef	notdef
#define	__STDC__	0
#endif
